@@ -1,16 +1,21 @@
 """CLI for reportseff."""
-from shutil import which
+
+from __future__ import annotations
+
 import sys
-from typing import Any, Dict, List, Tuple
+from shutil import which
+from typing import Any
 
 import click
 
 from . import __version__
 from .db_inquirer import BaseInquirer, SacctInquirer
 from .job_collection import JobCollection
-from .output_renderer import OutputRenderer
+from .output_renderer import OutputRenderer, RenderOptions
 from .parameters import ReportseffParameters
 
+MAX_ENTRIES_TO_ECHO = 20
+
 
 @click.command()
 @click.option(
@@ -65,6 +70,12 @@ from .parameters import ReportseffParameters
     default="",
     help="Only include jobs with the specified partition",
 )
+@click.option(
+    "-M",
+    "--cluster",
+    default="",
+    help="Select specific cluster, for multi-cluster system only",
+)
 @click.option(
     "--extra-args",
     default="",
@@ -114,7 +125,23 @@ from .parameters import ReportseffParameters
     "-p",
     is_flag=True,
     default=False,
-    help="Output will be '|' delmited without a '|' at the end.",
+    help="Output will be delmited without a delimiter at the end. "
+    "Delimiter is by default '|', to change it see --delimiter flag.",
+)
+@click.option(
+    "--delimiter",
+    "-d",
+    default="|",
+    help="Delimiter used for parsable output. The default default "
+    "delimiter is '|' when --parsable is specified. "
+    "This option is ignored if --parsable or -p is not specified.",
+)
+@click.option(
+    "--array-min-size",
+    default=0,
+    type=int,
+    help="Only include array jobs with at least this many tasks. "
+    "Non-array jobs are always included. Set to 0 to include all jobs (default).",
 )
 @click.version_option(version=__version__)
 @click.argument("jobs", nargs=-1)
@@ -124,13 +151,13 @@ def main(**kwargs: Any) -> None:
 
     output, entries = get_jobs(args)
 
-    if entries > 20:
+    if entries > MAX_ENTRIES_TO_ECHO:
         click.echo_via_pager(output, color=args.color)
     else:
         click.echo(output, color=args.color)
 
 
-def get_jobs(args: ReportseffParameters) -> Tuple[str, int]:
+def get_jobs(args: ReportseffParameters) -> tuple[str, int]:
     """Helper method to get jobs from db_inquirer.
 
     Returns:
@@ -146,7 +173,11 @@ def get_jobs(args: ReportseffParameters) -> Tuple[str, int]:
         job_collection.set_custom_seff_format(args.slurm_format)
 
     inquirer, renderer = get_implementation(
-        args.format_str, args.node, args.node_and_gpu, args.parsable
+        args.format_str,
+        node=args.node,
+        node_and_gpu=args.node_and_gpu,
+        parsable=args.parsable,
+        delimiter=args.delimiter,
     )
 
     inquirer.set_state(args.state)
@@ -156,6 +187,7 @@ def get_jobs(args: ReportseffParameters) -> Tuple[str, int]:
     inquirer.set_until(args.until)
 
     inquirer.set_partition(args.partition)
+    inquirer.set_cluster(args.cluster)
 
     inquirer.set_extra_args(args.extra_args)
 
@@ -176,15 +208,25 @@ def get_jobs(args: ReportseffParameters) -> Tuple[str, int]:
         sys.exit(1)
 
     job_collection.set_partition_limits(inquirer.get_partition_timelimits())
-    db_output = get_db_output(inquirer, renderer, job_collection, args.debug)
-    for entry in db_output:
-        try:
+    db_output = get_db_output(
+        inquirer,
+        renderer,
+        job_collection,
+        debug=args.debug,
+    )
+    entry = None
+    try:
+        for entry in db_output:
             job_collection.process_entry(entry, add_job=add_jobs)
-        except Exception as error:
-            click.echo(f"Error processing entry: {entry}", err=True)
-            raise error
+    except Exception:
+        click.echo(f"Error processing entry: {entry}", err=True)
+        raise
 
-    found_jobs = job_collection.get_sorted_jobs(args.modified_sort)
+    # Apply array size filtering if specified
+    if args.array_min_size > 0:
+        job_collection.filter_by_array_size(args.array_min_size)
+
+    found_jobs = job_collection.get_sorted_jobs(change_sort=args.modified_sort)
     found_jobs = [j for j in found_jobs if j.state]
 
     return renderer.format_jobs(found_jobs), len(found_jobs)
@@ -192,14 +234,20 @@ def get_jobs(args: ReportseffParameters) -> Tuple[str, int]:
 
 def get_implementation(
     format_str: str,
+    *,
     node: bool = False,
     node_and_gpu: bool = False,
     parsable: bool = False,
-) -> Tuple[BaseInquirer, OutputRenderer]:
+    delimiter: str = " ",
+) -> tuple[BaseInquirer, OutputRenderer]:
     """Get system-specific objects.
 
     Args:
         format_str: the formatting options specified by user
+        node: control if node-level stats are displayed
+        node_and_gpu: control if node and gpu stats are displayed
+        parsable: produce output with a delimiter separating columns
+        delimiter: delimiter used for parsable output
 
     Returns:
         A db_inqurirer
@@ -209,10 +257,13 @@ def get_implementation(
         inquirer = SacctInquirer()
         renderer = OutputRenderer(
             inquirer.get_valid_formats(),
+            RenderOptions(
+                node=node or node_and_gpu,
+                gpu=node_and_gpu,
+                parsable=parsable,
+                delimiter=delimiter,
+            ),
             format_str,
-            node=node or node_and_gpu,
-            gpu=node_and_gpu,
-            parsable=parsable,
         )
     else:
         click.secho("No supported scheduling systems found!", fg="red", err=True)
@@ -225,8 +276,9 @@ def get_db_output(
     inquirer: BaseInquirer,
     renderer: OutputRenderer,
     job_collection: JobCollection,
+    *,
     debug: bool,
-) -> List[Dict[str, str]]:
+) -> list[dict[str, str]]:
     """Get output from inquirer.
 
     Returns:
